/*!
 * SPDX-License-Identifier: Unlicense
 * This file was automatically generated by https://github.com/yt-dlp/ejs
 */
const lib = {
  meriyah: await import('meriyah@6.1.4'),
  astring: await import('astring@1.9.0'),
};
export { lib };
